Import("display.Bitmap");
Import("display.Loader");
Import("desktop.NativeApplication");
Import("display.NativeMenu");
Import("display.NativeMenuItem");
Import("display.NativeWindowInitOptions");
Import("display.NativeWindowType");
Import("events.NativeWindowDisplayStateEvent");
Import("events.Event");
Import("events.MouseEvent");
Import("html.HTMLLoader");
Import("geom.Rectangle");
Import("net.URLRequest");


application = NativeApplication.nativeApplication;

var iconLoadComplete = function(event){ 
    application.icon.bitmaps = [event.target.content.bitmapData]; 
} 
application.icon.menu = new NativeMenu();
//application.icon.tooltip = "AirDo";     
application.autoExit = false;

//Application Context 
var AppContext={};

//Contains array of windows 
AppContext.Windows={};
AppContext.ActiveWindow=null;

//Mode object
AppContext.mode={};

//Modes
AppContext.mode.TOGGLE="Visible";
AppContext.mode.WEB_SLIM="Slim Web View";
AppContext.mode.WEB="Full Web View";
AppContext.mode.NATIVE="AirDo Applicaiton";
//Menu Items representing modes
AppContext.mode.MenuItems={};

AppContext.mode.current=Preferences.getd("CurrentWindow",AppContext.mode.WEB_SLIM);
AppContext.AlwaysOnTop=Preferences.getd("AlwaysOnTop", true);
AppContext.AlwaysOnTopMenuItem=null;

var iconLoad = new Loader(); 
var iconMenu = generateMenu(); 



if (NativeApplication.supportsSystemTrayIcon) { 
		application.autoExit = false; 
        iconLoad.contentLoaderInfo.addEventListener(Event.COMPLETE,iconLoadComplete); 
        iconLoad.load(new URLRequest("../images/icons/icon-16.png")); 
        application.icon.tooltip = "AirDo"; 
        application.icon.menu = iconMenu; 
}

if (NativeApplication.supportsDockIcon) { 
	iconLoad.contentLoaderInfo.addEventListener(Event.COMPLETE,iconLoadComplete); 
	iconLoad.load(new URLRequest("../images/icons/icon-128.png")); 
    application.icon.menu = iconMenu; 
} 

function updateAlwaysOnTop(){
	AppContext.AlwaysOnTopMenuItem.checked=AppContext.AlwaysOnTop;
	if (AppContext.ActiveWindow){
		AppContext.ActiveWindow.stage.nativeWindow.alwaysInFront=AppContext.AlwaysOnTop;
	}	
}
function onAlwaysOnTop(event){
	//trace("Changing Always On Top From "+AppContext.AlwaysOnTop);
	AppContext.AlwaysOnTop =! AppContext.AlwaysOnTop;
	Preferences.set("AlwaysOnTop",AppContext.AlwaysOnTop);
	//trace("Changing Always On Top To "+AppContext.AlwaysOnTop);
	updateAlwaysOnTop();
}
function onShowConfigure(event){
	//trace("Configuration...");
	showWindow("Configuration");
}

function onExitRequest(event){ 
	application.icon.bitmaps = []; 
	application.exit();
}


function changeMode(){
	//trace("Changing Mode to.... "+AppContext.mode.current);
	
	for (var item in AppContext.mode.MenuItems){
		if (item == AppContext.mode.current){
			AppContext.mode.MenuItems[item].checked=true;
			showWindow(AppContext.mode.current, true);
			
		}else{
			AppContext.mode.MenuItems[item].checked=false;
		}	
	}
	setVisible(true);
}

function onChangeMode(event){	
	AppContext.mode.current=event.target.label;
	Preferences.set("CurrentWindow",AppContext.mode.current);
	changeMode();
}
function setVisible(isVisible){
	if (AppContext.ActiveWindow){
		AppContext.ActiveWindow.stage.nativeWindow.visible= isVisible;
		AppContext.mode.MenuItems[AppContext.mode.TOGGLE].checked=isVisible;
	}
}
function toggleVisible(){
	if (AppContext.ActiveWindow){
		setVisible(!AppContext.ActiveWindow.stage.nativeWindow.visible);
	}
}

function createWindow(name, x, y, width, height,URL){
	//trace("Create Window .... "+name);
	var options = new NativeWindowInitOptions();
	options.maximizable = true;
	options.minimizable = true;
	//options.type = NativeWindowType.LIGHTWEIGHT;
	//options.type = NativeWindowType.UTILITY;
	options.type = NativeWindowType.NORMAL;
	
	//transparent windows must have the systemChrome set to none
	//options.systemChrome = air.NativeWindowSystemChrome.NONE;

	//trace("Options done...");
	var bounds=new Rectangle(x, y, width, height);
	var htmlLoader = HTMLLoader.createRootWindow(false, options, true, bounds); //visible, options, scrollBarsVisible, bounds
	
	//trace("Loader ...");					
	htmlLoader.stage.nativeWindow.alwaysInFront = AppContext.AlwaysInFront;
	htmlLoader.stage.nativeWindow.title=name;
	htmlLoader.navigateInSystemBrowser = false;
	//trace("Loader done...");
		
	htmlLoader.addEventListener(Event.COMPLETE, function(event){
		// Remove the complete event, as it is also called when the window is closed
		//trace("Created window..."); 
		event.target.removeEventListener(Event.COMPLETE, arguments.callee );
			// Put the title and message into the elements and also resize the window to have
			 // the entire message in view
			//self.injectMessage();

			// Ready to make the window visible
			event.target.stage.nativeWindow.visible = true;
/*
			event.target.stage.nativeWindow.addEventListener(NativeWindowDisplayStateEvent.CLOSING, function(event){
				//trace("Unloading...");
				for(windowId in AppContext.Windows){
					if (AppContext.Windows[windowId]==event.target){
						//trace("Removing this window from list");
						AppContext.Windows[windowId]=null;
						if(AppContext.CurrentWindow==event.target){
							AppContext.CurrentWindow=null;
							//trace("Removing this window from being current");
									
						}
					}
				}
			});
*/
	});
	// When the paintsDefaultBackground = false, the runtime makes the body background transparent.
	//htmlLoader.paintsDefaultBackground = false;

	if (!URL){
			template="<html>" +
				"<head>" +
				"<script src=\"../javascript/jquery-1.3.2.min.js\" type=\"text/javascript\" charset=\"utf-8\"></script>"+
				"</head>" + 
				"<body style='opacity:1'><div style='background-color:white;'><h1>"+name+"</h1></div></body>"+ 
		   "</html>";
		// Inject the template in the htmlLoader
		htmlLoader.loadString(template);
	}else{
		//trace("Lets load...");
		htmlLoader.load(new URLRequest(URL));
		//trace("Loaded...");
	}
			
	/*					
	// We should close all the notifications when the window is closing
	this.windowCloseEvent = function(){
		self.close();
	}
	*/
	if (NativeApplication.supportsSystemTrayIcon) {			
			htmlLoader.addEventListener(NativeWindowDisplayStateEvent.DISPLAY_STATE_CHANGE, function(event) {
				//trace("Display State Change");
				if (event.afterDisplayState == NativeWindowDisplayState.MINIMIZED) {
					setVisible(false);
				}
			});
	}else{
		if (NativeApplication.supportsDockIcon) {			
			htmlLoader.addEventListener(NativeWindowDisplayStateEvent.DISPLAY_STATE_CHANGE, function(event) {
				//trace("Display State Change");
				if (event.afterDisplayState == NativeWindowDisplayState.MINIMIZED) {
					setVisible(false);
				}
			});
		}
	}
	return htmlLoader;	
}


function createLightweightWindow(name, x, y, width, height,URL){
	//trace("Create Window .... "+name);
	var options = new NativeWindowInitOptions();
	options.transparent = true;
	options.maximizable = true;
	options.minimizable = true;
	options.type = NativeWindowType.LIGHTWEIGHT;
	
	//transparent windows must have the systemChrome set to none
	options.systemChrome = air.NativeWindowSystemChrome.NONE;

	//
	var bounds=new Rectangle(x, y, width, height);
	var htmlLoader = HTMLLoader.createRootWindow(false, options, true, bounds); //visible, options, scrollBarsVisible, bounds
						
	htmlLoader.stage.nativeWindow.alwaysInFront = true;
	htmlLoader.navigateInSystemBrowser = false;
	
		
	htmlLoader.addEventListener(Event.COMPLETE, function(event){
		// Remove the complete event, as it is also called when the window is closed
		//trace("Created window..."); 
		event.target.removeEventListener(Event.COMPLETE, arguments.callee );
			// Put the title and message into the elements and also resize the window to have
			 // the entire message in view
			//self.injectMessage();

			// Ready to make the window visible
			event.target.stage.nativeWindow.visible = true;
			});

	// When the paintsDefaultBackground = false, the runtime makes the body background transparent.
	htmlLoader.paintsDefaultBackground = false;

	if (!URL){
			template="<html>" +
				"<head>" +
				"<script src=\"../javascript/jquery-1.3.2.min.js\" type=\"text/javascript\" charset=\"utf-8\"></script>"+
				"</head>" + 
				"<body style='opacity:1'><div style='background-color:white;'><h1>"+name+"</h1></div></body>"+ 
		   "</html>";
		// Inject the template in the htmlLoader
		htmlLoader.loadString(template);
	}else{
		htmlLoader.load(new URLRequest(URL));
	
	}
	/*
	// add the onclick handler
	htmlLoader.addEventListener("click", function(){
		if(self.onclick){
			self.onclick();
		}
	});
	*/	
			
	// We should close all the notifications when the window is closing
	this.windowCloseEvent = function(){
		self.close();
	}
	return htmlLoader;
//	window.addEventListener('unload', this.windowCloseEvent);
}

function showWindow(windowName, createIfNone){	
	//trace("Show Window .... "+windowName);
	var desiredWindow=AppContext.Windows[windowName];
	
	if (! desiredWindow || desiredWindow.stage.nativeWindow.closed){
		//trace("No Window Named .... "+windowName);
		switch(windowName){
			case AppContext.mode.WEB_SLIM:
				var bounds=Preferences.WebSlimBounds;
				var URL=Preferences.WebSlimURL;
				desiredWindow=createWindow(windowName,bounds.x,bounds.y,bounds.width, bounds.height, URL);
				break;
			case AppContext.mode.WEB:
				var bounds=Preferences.WebBounds;
				var URL=Preferences.WebURL;
				desiredWindow=createWindow(windowName,bounds.x,bounds.y,bounds.width, bounds.height, URL);
				break;
			default:
				if (createIfNone){
					desiredWindow=createLightweightWindow(windowName, 100,100,400,400);
				}
				break;				
		}
		if(desiredWindow){
			AppContext.Windows[windowName]=desiredWindow;
		}
	}
	if(desiredWindow){
		AppContext.ActiveWindow=desiredWindow;
	}
	
	//Show the right window
	for(windowId in AppContext.Windows){
		if (AppContext.Windows[windowId]==AppContext.ActiveWindow){
			AppContext.Windows[windowId].stage.nativeWindow.visible = true;
		}else{
			AppContext.Windows[windowId].stage.nativeWindow.visible = false;
		}
		AppContext.Windows[windowId].stage.nativeWindow.alwaysInFront = AppContext.AlwaysOnTop;		
	}
	
}

function generateMenu(){
	var menu=new NativeMenu();
	
	AppContext.mode.MenuItems={};
	
	function separator(){
		menu.addItem(new NativeMenuItem("", true));
	}
	
	
	AppContext.mode.MenuItems[AppContext.mode.TOGGLE] = menu.addItem(new NativeMenuItem(AppContext.mode.TOGGLE));
	AppContext.mode.MenuItems[AppContext.mode.TOGGLE].addEventListener(Event.SELECT, toggleVisible);
	separator();
	var modeMenuItem=menu.addItem(new NativeMenuItem("Mode:"));
	modeMenuItem.enabled=false;
	separator();
	// Mode Selection
	AppContext.mode.MenuItems[AppContext.mode.WEB_SLIM] = menu.addItem(new NativeMenuItem(AppContext.mode.WEB_SLIM));
	AppContext.mode.MenuItems[AppContext.mode.WEB_SLIM].addEventListener(Event.SELECT, onChangeMode);
	
	AppContext.mode.MenuItems[AppContext.mode.WEB]= menu.addItem(new NativeMenuItem(AppContext.mode.WEB));
	AppContext.mode.MenuItems[AppContext.mode.WEB].addEventListener(Event.SELECT, onChangeMode);
	
	// Removing NATIVE app for now
	//AppContext.mode.MenuItems[AppContext.mode.NATIVE] = menu.addItem(new NativeMenuItem(AppContext.mode.NATIVE));
	//AppContext.mode.MenuItems[AppContext.mode.NATIVE].addEventListener(Event.SELECT, onChangeMode);
	

	//Configuration
	// Removing configuration for now
	separator();
	//var configure = menu.addItem(new NativeMenuItem("Configure"));
	//configure.addEventListener(Event.SELECT, onShowConfigure);	
	AppContext.AlwaysOnTopMenuItem = menu.addItem(new NativeMenuItem("Always On Top"));
	AppContext.AlwaysOnTopMenuItem.addEventListener(Event.SELECT, onAlwaysOnTop);	
	updateAlwaysOnTop();
	
	separator();
	//Exit
	var exitCommand = menu.addItem(new NativeMenuItem("Exit"));
	exitCommand.addEventListener(Event.SELECT,onExitRequest);
	
	//Set mode
	changeMode();
	
	return menu;
}

application.icon.addEventListener(MouseEvent.CLICK, toggleVisible);
setVisible(true);